#! /usr/bin/env python

##------------------------------------------------------------------------------
##
## Name: mkstiscsum  rootname
##
##  Runs a minimal version of CALSTIS to produce the csums from the rawfits and
##  wavecals associated with that rootname.
##
## Written by Phil Hodge  August 2010
##
## mm/dd/yy PR    who        Description
## -------- ----- ---------- ---------------------------------------------------
## 09/10/10 61157 Mary       Initial Version by Phil Hodge
## 09/21/10 61157 MSwam      Added more exit codes a test for no files found
## 11/30/10 61157 Sherbert   Add a few messages, change temp name slightly
## 12/14/11 67718 Mary       Move the test for duplication closer to the write
##
##------------------------------------------------------------------------------

import commands
import os, os.path
import sys
import numpy as np
import pyfits
from opusutil import PrintMsg

MAX_ENVIRON = 1000      # maximum depth of environment variables for path
nm = os.path.basename(sys.argv[0]) 

def mkstiscsum (input):
    """Create 'csum' file(s) for STIS raw or wav data.

    @param input: name of input file
    @type input: string

    @return: status code returned by calstis1, or 0 if OK
    @rtype: int
    """

    detector = getDetectorName (input)
    if detector == "CCD":
        return 0
    if detector != "FUV-MAMA" and detector != "NUV-MAMA":
        raise RuntimeError, "invalid detector name '%s'" % detector

    if input.endswith ("_raw.fits"):
        i = input.rfind ("_raw.fits")
        temp = input[0:i] + "_rcsum_tmp.fits"
    elif input.endswith ("_wav.fits"):
        i = input.rfind ("_wav.fits")
        temp = input[0:i] + "_wcsum_tmp.fits"
    else:
        raise RuntimeError, "suffix in file name '%s' was not expected" % input

    temp = os.path.basename (temp)
    if os.access (temp, os.F_OK):
        print "Temporary file '%s' already exists; skipping ..." % temp
        return 0

    path = getPath()
    # run calstis1
    cmd = path + " -lors -glin " + input + " " + temp
    (status, trailer) = commands.getstatusoutput (cmd)
    if status:
        print trailer
        print "### status =", status, "but we are ignorning it"
        return status

    ifd = pyfits.open (temp, mode="readonly")
    template_phdr = ifd[0].header
    deletePhdrKwds (template_phdr)

    nextend = len (ifd) - 1
    nimsets = nextend // 3
    if nextend != nimsets * 3:
        raise RuntimeError, "number of extensions is %d," \
                " should be a multiple of 3" % nextend

    for i in range (nimsets):
        hdunum = 1 + i * 3
        writeCsum (template_phdr, ifd[hdunum], detector)

    ifd.close()

    msg = "os.remov (" + temp + ")"
    PrintMsg( 'D', msg, nm )
    os.remove (temp)

    return 0

def getDetectorName (input):
    """Get the value of the DETECTOR keyword from the primary header.

    @param input: name of input file
    @type input: string

    @return: detector name, or "missing" if the keyword was not found
    @rtype: string
    """

    fd = pyfits.open (input, mode="readonly")
    detector = fd[0].header.get ("detector", "missing")
    fd.close()

    return detector

def getPath():
    """Get the name (including directory) of the calstis1 executable.

    @return: name of file, including directory
    @rtype: string
    """

    if os.environ.has_key ("STSDASDISK") and \
       os.environ.has_key ("IRAFARCH"):
        stsdasdisk = os.environ["STSDASDISK"]
        irafarch = os.environ["IRAFARCH"]
        bindir = "bin." + irafarch
        path = os.path.join (stsdasdisk, bindir)
        path_save = path        # save for possible error message
        niter = 0               # initialize
        done = False
        while not done:
            fullpath = os.path.expandvars (path)
            if fullpath == path:
                done = True
            path = fullpath
            niter += 1
            if niter > MAX_ENVIRON:
                raise RuntimeError, "couldn't expand environment variables" \
                        " in pathname '%s'" % path_save
    else:
        if not os.environ.has_key ("STSDASDISK"):
            print "Warning:  environment variable STSDASDISK is not defined."
        if not os.environ.has_key ("IRAFARCH"):
            print "Warning:  environment variable IRAFARCH is not defined."
        path = "."

    return os.path.join (path, "cs1.e")

def deletePhdrKwds (template_phdr):
    """Delete some keywords from the primary header.

    @param template_phdr: primary header from input file (modified in-place)
    @type template_phdr: pyfits Header object

    Keywords TDATEOBS, TTIMEOBS, TEXPSTRT, TEXPEND, TEXPTIME will be deleted.
    """

    for keyword in ["tdateobs", "ttimeobs", "texpstrt", "texpend", "texptime"]:
        if template_phdr.has_key (keyword):
            del (template_phdr[keyword])

def writeCsum (template_phdr, hdu, detector):
    """Write a csum file for the current extension.

    @param template_phdr: primary header from input file
    @type template_phdr: pyfits Header object
    @param hdu: the current header/data unit
    @type hdu: pyfits ImageHDU object
    @param detector: detector name
    @type detector: string
    """

    hdr = hdu.header
    extname = hdr.get ("extname", "missing")
    if extname != "SCI":
        raise RuntimeError, "unexpected EXTNAME '%s'" % extname

    primary_hdu = pyfits.PrimaryHDU (header=template_phdr)
    ofd = pyfits.HDUList (primary_hdu)
    ofd.append (hdu)
    binData (ofd)       # bin to 1024x1024 if not already done

    expname = ofd[1].header.get ("expname", "missing")
    phdr = ofd[0].header
    if detector == "FUV-MAMA":
        output = expname + "_csum_f.fits"
    elif detector == "NUV-MAMA":
        output = expname + "_csum_n.fits"
    else:
        raise RuntimeError, "invalid detector name '%s'" % detector
    msg = "output is " + output
    PrintMsg( 'I', msg, nm )

    ### PR 67718 os.access test for file already exists originally here.
    ###   moving it down before the ofd.writeto call
    ###   to reduce further file exists errors.
    
    phdr.update ("filename", output)
    phdr.update ("filetype", "STIS SUM FILE")

    # compute counts, add keyword to extension header
    counts = hdu.data.sum (dtype=np.float64)
    ofd[1].header.update ("counts", counts)

    if os.access (output, os.F_OK):
        print "Cumulative image file '%s' already exists;" \
              " will not overwrite." % output
        return

    ofd.writeto (output)

def binData (ofd):
    """Bin the data block to 1024x1024 if necessary.

    @param ofd: header and data for output, may be modified in-place
    @type ofd: pyfits HDUList object
    """

    if ofd[1].data is None:
        return

    shape = ofd[1].data.shape
    modified = False            # initial value
    if shape[0] != 1024 or shape[1] != 1024:
        data = ofd[1].data

    if shape[1] != 1024:
        if shape[1] != 2048:
            raise RuntimeError, "invalid X dimension %d for data" % shape[1]
        data = data[:,0:2047:2] + data[:,1:2048:2]
        modified = True

    if shape[0] != 1024:
        if shape[0] != 2048:
            raise RuntimeError, "invalid Y dimension %d for data" % shape[0]
        data = data[0:2047:2] + data[1:2048:2]
        modified = True

    if modified:
        ofd[1].data = data

def main (rootname):

    found_files = 0
    for tail in ["_raw.fits", "_wav.fits"]:
        input = rootname + tail
        msg = "input is " + input
        PrintMsg( 'I', msg, nm )
        if os.access (input, os.R_OK):
            mkstiscsum (input)
            found_files = 1

    if not found_files:
        print "No files found to run against for "+rootname
        sys.exit(1)
      

if __name__ == "__main__":

    if len (sys.argv) != 2:
        print "syntax:  mkstiscsum.py rootname"
        sys.exit(-1)

    main (sys.argv[1])
    sys.exit(0)
